@@ -1,9 +1,9 @@
-;;; org-mobile.el --- Code for asymmetric sync with a mobile device
-;; Copyright (C) 2009-2014 Free Software Foundation, Inc.
+;;; org-mobile.el --- Code for Asymmetric Sync With a Mobile Device -*- lexical-binding: t; -*-
+;; Copyright (C) 2009-2018 Free Software Foundation, Inc.
 ;;
 ;; Author: Carsten Dominik <carsten at orgmode dot org>
 ;; Keywords: outlines, hypermedia, calendar, wp
-;; Homepage: http://orgmode.org
+;; Homepage: https://orgmode.org
 ;;
 ;; This file is part of GNU Emacs.
 ;;
@@ -18,43 +18,44 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;;; Commentary:
 ;;
-;; This file contains the code to interact with Richard Moreland's iPhone
-;; application MobileOrg, as well as with the Android version by Matthew Jones.
-;; This code is documented in Appendix B of the Org-mode manual.  The code is
-;; not specific for the iPhone and Android - any external
-;; viewer/flagging/editing application that uses the same conventions could
-;; be used.
+;; This file contains the code to interact with a mobile application,
+;; such as Richard Moreland's iPhone application MobileOrg, or the
+;; Android version by Matthew Jones.  This code is documented in
+;; Appendix B of the Org manual.  The code is not specific for the
+;; iPhone and Android - any external viewer/flagging/editing
+;; application that uses the same conventions could be used.
 
 (require 'org)
 (require 'org-agenda)
+(require 'cl-lib)
+
+(defvar org-agenda-keep-restricted-file-list)
+
 ;;; Code:
 
-(eval-when-compile (require 'cl))
-
-(declare-function org-pop-to-buffer-same-window
-		  "org-compat" (&optional buffer-or-name norecord label))
-
 (defgroup org-mobile nil
   "Options concerning support for a viewer/editor on a mobile device."
   :tag "Org Mobile"
   :group 'org)
 
 (defcustom org-mobile-files '(org-agenda-files)
-  "Files to be staged for MobileOrg.
+  "Files to be staged for the mobile application.
+
 This is basically a list of files and directories.  Files will be staged
-directly.  Directories will be search for files with the extension `.org'.
+directly.  Directories will be search for files with the extension \".org\".
 In addition to this, the list may also contain the following symbols:
 
-org-agenda-files
+`org-agenda-files'
      This means include the complete, unrestricted list of files given in
      the variable `org-agenda-files'.
-org-agenda-text-search-extra-files
+
+`org-agenda-text-search-extra-files'
      Include the files given in the variable
      `org-agenda-text-search-extra-files'"
   :group 'org-mobile
@@ -85,12 +86,14 @@ org-agenda-text-search-extra-files
 
 (defcustom org-mobile-use-encryption nil
   "Non-nil means keep only encrypted files on the WebDAV server.
+
 Encryption uses AES-256, with a password given in
-`org-mobile-encryption-password'.
-When nil, plain files are kept on the server.
-Turning on encryption requires to set the same password in the MobileOrg
-application.  Before turning this on, check of MobileOrg does already
-support it - at the time of this writing it did not yet."
+`org-mobile-encryption-password'.  When nil, plain files are kept
+on the server.
+
+Turning on encryption requires setting the same password in the
+mobile application.  Before turning this on, check if the mobile
+application does support it."
   :group 'org-mobile
   :version "24.1"
   :type 'boolean)
@@ -105,9 +108,10 @@ You might want to put this file into a directory where only you have access."
 
 (defcustom org-mobile-encryption-password ""
   "Password for encrypting files uploaded to the server.
+
 This is a single password which is used for AES-256 encryption.  The same
-password must also be set in the MobileOrg application.  All Org files,
-including mobileorg.org will be encrypted using this password.
+password must also be set in the mobile application.  All Org files,
+including \"mobileorg.org\" will be encrypted using this password.
 
 SECURITY CONSIDERATIONS:
 
@@ -130,12 +134,12 @@ session."
   (or (org-string-nw-p org-mobile-encryption-password)
       (org-string-nw-p org-mobile-encryption-password-session)
       (setq org-mobile-encryption-password-session
-	    (read-passwd "Password for MobileOrg: " t))))
+	    (read-passwd "Password for mobile application: " t))))
 
 (defcustom org-mobile-inbox-for-pull "~/org/from-mobile.org"
   "The file where captured notes and flags will be appended to.
 During the execution of `org-mobile-pull', the file
-`org-mobile-capture-file' will be emptied it's contents have
+`org-mobile-capture-file' is emptied as soon as its contents have
 been appended to the file given here.  This file should be in
 `org-directory', and not in the staging area or on the web server."
   :group 'org-mobile
@@ -143,23 +147,25 @@ been appended to the file given here.  This file should be in
 
 (defconst org-mobile-capture-file "mobileorg.org"
   "The capture file where the mobile stores captured notes and flags.
-This should not be changed, because MobileOrg assumes this name.")
+This must not be changed, because the mobile application assumes this name.")
 
 (defcustom org-mobile-index-file "index.org"
-  "The index file with links to all Org files that should be loaded by MobileOrg.
-Relative to `org-mobile-directory'.  The Address field in the MobileOrg setup
-should point to this file."
+  "Index file with links to all Org files.
+It should be loaded by the mobile application.  The file name is
+relative to `org-mobile-directory'.  The \"Address\" field in the
+mobile application setup should point to this file."
   :group 'org-mobile
   :type 'file)
 
 (defcustom org-mobile-agendas 'all
-  "The agendas that should be pushed to MobileOrg.
+  "The agendas that should be pushed to the mobile application.
+
 Allowed values:
 
-default  the weekly agenda and the global TODO list
-custom   all custom agendas defined by the user
-all      the custom agendas and the default ones
-list     a list of selection key(s) as string."
+`default'  the weekly agenda and the global TODO list
+`custom'   all custom agendas defined by the user
+`all'      the custom agendas and the default ones
+`list'     a list of selection key(s) as string."
   :group 'org-mobile
   :version "24.1"
   :type '(choice
@@ -192,27 +198,6 @@ the editing types for which the mobile version should always dominate."
 	       (const heading)
 	       (const body))))
 
-(defcustom org-mobile-action-alist
-  '(("edit" . (org-mobile-edit data old new)))
-  "Alist with flags and actions for mobile sync.
-When flagging an entry, MobileOrg will create entries that look like
-
-  * F(action:data)  [[id:entry-id][entry title]]
-
-This alist defines that the ACTION in the parentheses of F() should mean,
-i.e. what action should be taken.  The :data part in the parenthesis is
-optional.  If present, the string after the colon will be passed to the
-action form as the `data' variable.
-The car of each elements of the alist is an actions string.  The cdr is
-an Emacs Lisp form that will be evaluated with the cursor on the headline
-of that entry.
-
-For now, it is not recommended to change this variable."
-  :group 'org-mobile
-  :type '(repeat
-	  (cons (string :tag "Action flag")
-		(sexp   :tag "Action form"))))
-
 (defcustom org-mobile-checksum-binary (or (executable-find "shasum")
 					  (executable-find "sha1sum")
 					  (executable-find "md5sum")
@@ -249,6 +234,25 @@ by the mobile device, this hook should be used to copy the emptied
 capture file `mobileorg.org' back to the WebDAV directory, for example
 using `rsync' or `scp'.")
 
+(defconst org-mobile-action-alist '(("edit" . org-mobile-edit))
+  "Alist with flags and actions for mobile sync.
+
+When flagging an entry, the mobile application creates entries
+that look like
+
+  * F(action:data)  [[id:entry-id][entry title]]
+
+This alist defines that the ACTION in the parentheses of F()
+should mean, i.e. what action should be taken.  The :data part in
+the parenthesis is optional.  If present, the string after the
+colon will be passed to the action function as the first argument
+variable.
+
+The car of each elements of the alist is an actions string.  The
+cdr is a function that is called with the cursor on the headline
+of that entry.  It should accept three arguments, the :data part,
+the old and new values for the entry.")
+
 (defvar org-mobile-last-flagged-files nil
   "List of files containing entries flagged in the latest pull.")
 
@@ -313,40 +317,34 @@ Also exclude files matching `org-mobile-files-exclude-regexp'."
 This will create the index file, copy all agenda files there, and also
 create all custom agenda views, for upload to the mobile phone."
   (interactive)
-  (let ((a-buffer (get-buffer org-agenda-buffer-name)))
-    (let ((org-agenda-curbuf-name org-agenda-buffer-name)
-	  (org-agenda-buffer-name "*SUMO*")
-	  (org-agenda-tag-filter org-agenda-tag-filter)
-	  (org-agenda-redo-command org-agenda-redo-command))
-      (save-excursion
-	(save-restriction
-	  (save-window-excursion
-	    (run-hooks 'org-mobile-pre-push-hook)
-	    (org-mobile-check-setup)
-	    (org-mobile-prepare-file-lists)
-	    (message "Creating agendas...")
-	    (let ((inhibit-redisplay t)
-		  (org-agenda-files (mapcar 'car org-mobile-files-alist)))
-	      (org-mobile-create-sumo-agenda))
-	    (message "Creating agendas...done")
-	    (org-save-all-org-buffers) ; to save any IDs created by this process
-	    (message "Copying files...")
-	    (org-mobile-copy-agenda-files)
-	    (message "Writing index file...")
-	    (org-mobile-create-index-file)
-	    (message "Writing checksums...")
-	    (org-mobile-write-checksums)
-	    (run-hooks 'org-mobile-post-push-hook))))
-      (setq org-agenda-buffer-name org-agenda-curbuf-name
-	    org-agenda-this-buffer-name org-agenda-curbuf-name))
-    (redraw-display)
-    (when (buffer-live-p a-buffer)
-      (if (not (get-buffer-window a-buffer))
-    	  (kill-buffer a-buffer)
-    	(let ((cw (selected-window)))
-    	  (select-window (get-buffer-window a-buffer))
-    	  (org-agenda-redo)
-    	  (select-window cw)))))
+  (let ((org-agenda-buffer-name "*SUMO*")
+	(org-agenda-tag-filter org-agenda-tag-filter)
+	(org-agenda-redo-command org-agenda-redo-command))
+    ;; Offer to save agenda-related buffers before pushing, preventing
+    ;; "Non-existent agenda file" prompt for lock files (see #19448).
+    (let ((agenda-buffers (org-buffer-list 'agenda)))
+      (save-some-buffers nil
+			 (lambda () (memq (current-buffer) agenda-buffers))))
+    (save-excursion
+      (save-restriction
+	(save-window-excursion
+	  (run-hooks 'org-mobile-pre-push-hook)
+	  (org-mobile-check-setup)
+	  (org-mobile-prepare-file-lists)
+	  (message "Creating agendas...")
+	  (let ((inhibit-redisplay t)
+		(org-agenda-files (mapcar 'car org-mobile-files-alist)))
+	    (org-mobile-create-sumo-agenda))
+	  (message "Creating agendas...done")
+	  (org-save-all-org-buffers) ; to save any IDs created by this process
+	  (message "Copying files...")
+	  (org-mobile-copy-agenda-files)
+	  (message "Writing index file...")
+	  (org-mobile-create-index-file)
+	  (message "Writing checksums...")
+	  (org-mobile-write-checksums)
+	  (run-hooks 'org-mobile-post-push-hook)))))
+  (org-agenda-maybe-redo)
   (message "Files for mobile viewer staged"))
 
 (defvar org-mobile-before-process-capture-hook nil
@@ -422,10 +420,10 @@ agenda view showing the flagged items."
   (let ((files-alist (sort (copy-sequence org-mobile-files-alist)
 			   (lambda (a b) (string< (cdr a) (cdr b)))))
 	(def-todo (default-value 'org-todo-keywords))
-	(def-tags (default-value 'org-tag-alist))
+	(def-tags org-tag-alist)
 	(target-file (expand-file-name org-mobile-index-file
 				       org-mobile-directory))
-	file link-name todo-kwds done-kwds tags entry kwds dwds twds)
+	todo-kwds done-kwds tags)
     (when (stringp (car def-todo))
       (setq def-todo (list (cons 'sequence def-todo))))
     (org-agenda-prepare-buffers (mapcar 'car files-alist))
@@ -435,35 +433,24 @@ agenda view showing the flagged items."
 		     (org-uniquify org-todo-keywords-for-agenda)))
     (setq tags (mapcar 'car (org-global-tags-completion-table
 			     (mapcar 'car files-alist))))
-    (with-temp-file
-	(if org-mobile-use-encryption
-	    org-mobile-encryption-tempfile
-	  target-file)
+    (with-temp-file (if org-mobile-use-encryption org-mobile-encryption-tempfile
+		      target-file)
       (insert "#+READONLY\n")
-      (while (setq entry (pop def-todo))
-	(setq kwds (mapcar (lambda (x) (if (string-match "(" x)
-					   (substring x 0 (match-beginning 0))
-					 x))
-			   (cdr entry)))
-	(insert "#+TODO: " (mapconcat 'identity kwds " ") "\n")
-	(setq dwds (or (member "|" kwds) (last kwds))
-	      twds (org-delete-all dwds kwds)
-	      todo-kwds (org-delete-all twds todo-kwds)
-	      done-kwds (org-delete-all dwds done-kwds)))
+      (dolist (entry def-todo)
+	(let ((kwds (mapcar (lambda (x)
+			      (if (string-match "(" x)
+				  (substring x 0 (match-beginning 0))
+				x))
+			    (cdr entry))))
+	  (insert "#+TODO: " (mapconcat #'identity kwds " ") "\n")
+	  (let* ((dwds (or (member "|" kwds) (last kwds)))
+		 (twds (org-delete-all dwds kwds)))
+	    (setq todo-kwds (org-delete-all twds todo-kwds))
+	    (setq done-kwds (org-delete-all dwds done-kwds)))))
       (when (or todo-kwds done-kwds)
 	(insert "#+TODO: " (mapconcat 'identity todo-kwds " ") " | "
 		(mapconcat 'identity done-kwds " ") "\n"))
-      (setq def-tags (mapcar
-		      (lambda (x)
-			(cond ((null x) nil)
-			      ((stringp x) x)
-			      ((eq (car x) :startgroup) "{")
-			      ((eq (car x) :endgroup) "}")
-			      ((eq (car x) :grouptags) nil)
-			      ((eq (car x) :newline) nil)
-			      ((listp x) (car x))))
-		      def-tags))
-      (setq def-tags (delq nil def-tags))
+      (setq def-tags (split-string (org-tag-alist-to-string def-tags t)))
       (setq tags (org-delete-all def-tags tags))
       (setq tags (sort tags (lambda (a b) (string< (downcase a) (downcase b)))))
       (setq tags (append def-tags tags nil))
@@ -472,11 +459,8 @@ agenda view showing the flagged items."
       (when (file-exists-p (expand-file-name
 			    org-mobile-directory "agendas.org"))
 	(insert "* [[file:agendas.org][Agenda Views]]\n"))
-      (while (setq entry (pop files-alist))
-	(setq file (car entry)
-	      link-name (cdr entry))
-	(insert (format "* [[file:%s][%s]]\n"
-			link-name link-name)))
+      (pcase-dolist (`(,_ . ,link-name) files-alist)
+	(insert (format "* [[file:%s][%s]]\n" link-name link-name)))
       (push (cons org-mobile-index-file (md5 (buffer-string)))
 	    org-mobile-checksum-files))
     (when org-mobile-use-encryption
@@ -497,7 +481,7 @@ agenda view showing the flagged items."
 	  (make-directory target-dir 'parents))
 	(if org-mobile-use-encryption
 	    (org-mobile-encrypt-and-move file target-path)
-	  (copy-file file target-path 'ok-if-exists))
+	  (copy-file file target-path 'ok-if-already-exists))
 	(setq check (shell-command-to-string
 		     (concat (shell-quote-argument org-mobile-checksum-binary)
 			     " "
@@ -662,7 +646,7 @@ The table of checksums is written to the file mobile-checksums."
 		     m 10 "   " 'planning)
 		    "\n")
 	    (when (setq id
-			(if (org-bound-and-true-p
+			(if (bound-and-true-p
 			     org-mobile-force-id-on-agenda-items)
 			    (org-id-get m 'create)
 			  (or (org-entry-get m "ID")
@@ -678,7 +662,7 @@ The table of checksums is written to the file mobile-checksums."
   (org-with-point-at pom
     (concat "olp:"
 	    (org-mobile-escape-olp (file-name-nondirectory buffer-file-name))
-	    "/"
+	    ":"
 	    (mapconcat 'org-mobile-escape-olp
 		       (org-get-outline-path)
 		       "/")
@@ -717,13 +701,13 @@ encryption program does not understand them."
   (let ((encfile (concat infile "_enc")))
     (org-mobile-encrypt-file infile encfile)
     (when outfile
-      (copy-file encfile outfile 'ok-if-exists)
+      (copy-file encfile outfile 'ok-if-already-exists)
       (delete-file encfile))))
 
 (defun org-mobile-encrypt-file (infile outfile)
   "Encrypt INFILE to OUTFILE, using `org-mobile-encryption-password'."
   (shell-command
-   (format "openssl enc -aes-256-cbc -salt -pass %s -in %s -out %s"
+   (format "openssl enc -md md5 -aes-256-cbc -salt -pass %s -in %s -out %s"
 	   (shell-quote-argument (concat "pass:"
 					 (org-mobile-encryption-password)))
 	   (shell-quote-argument (expand-file-name infile))
@@ -732,7 +716,7 @@ encryption program does not understand them."
 (defun org-mobile-decrypt-file (infile outfile)
   "Decrypt INFILE to OUTFILE, using `org-mobile-encryption-password'."
   (shell-command
-   (format "openssl enc -d -aes-256-cbc -salt -pass %s -in %s -out %s"
+   (format "openssl enc -md md5 -d -aes-256-cbc -salt -pass %s -in %s -out %s"
 	   (shell-quote-argument (concat "pass:"
 					 (org-mobile-encryption-password)))
 	   (shell-quote-argument (expand-file-name infile))
@@ -822,14 +806,14 @@ If BEG and END are given, only do this in that region."
 	(cnt-flag 0)
 	(cnt-error 0)
 	buf-list
-	id-pos org-mobile-error)
+	org-mobile-error)
 
     ;; Count the new captures
     (goto-char beg)
     (while (re-search-forward "^\\* \\(.*\\)" end t)
       (and (>= (- (match-end 1) (match-beginning 1)) 2)
 	   (not (equal (downcase (substring (match-string 1) 0 2)) "f("))
-	   (incf cnt-new)))
+	   (cl-incf cnt-new)))
 
     ;; Find and apply the edits
     (goto-char beg)
@@ -841,19 +825,21 @@ If BEG and END are given, only do this in that region."
 	       (id-pos (condition-case msg
 			   (org-mobile-locate-entry (match-string 4))
 			 (error (nth 1 msg))))
-	       (bos (point-at-bol))
+	       (bos (line-beginning-position))
 	       (eos (save-excursion (org-end-of-subtree t t)))
 	       (cmd (if (equal action "")
-			'(progn
-			   (incf cnt-flag)
-			   (org-toggle-tag "FLAGGED" 'on)
-			   (and note
-				(org-entry-put nil "THEFLAGGINGNOTE" note)))
-		      (incf cnt-edit)
+			(let ((note (buffer-substring-no-properties
+				     (line-beginning-position 2) eos)))
+			  (lambda (_data _old _new)
+			    (cl-incf cnt-flag)
+			    (org-toggle-tag "FLAGGED" 'on)
+			    (org-entry-put
+			     nil "THEFLAGGINGNOTE"
+			     (replace-regexp-in-string "\n" "\\\\n" note))))
+		      (cl-incf cnt-edit)
 		      (cdr (assoc action org-mobile-action-alist))))
-	       (note (and (equal action "")
-			  (buffer-substring (1+ (point-at-eol)) eos)))
-	       (org-inhibit-logging 'note) ;; Do not take notes interactively
+	       ;; Do not take notes interactively.
+	       (org-inhibit-logging 'note)
 	       old new)
 
 	  (goto-char bos)
@@ -866,11 +852,11 @@ If BEG and END are given, only do this in that region."
 	    (if (stringp id-pos)
 		(insert id-pos " ")
 	      (insert "BAD REFERENCE "))
-	    (incf cnt-error)
+	    (cl-incf cnt-error)
 	    (throw 'next t))
 	  (unless cmd
 	    (insert "BAD FLAG ")
-	    (incf cnt-error)
+	    (cl-incf cnt-error)
 	    (throw 'next t))
 	  (move-marker bos-marker (point))
 	  (if (re-search-forward "^** Old value[ \t]*$" eos t)
@@ -883,34 +869,28 @@ If BEG and END are given, only do this in that region."
 			 (progn (outline-next-heading)
 				(if (eobp) (org-back-over-empty-lines))
 				(point)))))
-	  (setq old (and old (if (string-match "\\S-" old) old nil)))
-	  (setq new (and new (if (string-match "\\S-" new) new nil)))
-	  (if (and note (> (length note) 0))
-	      ;; Make Note into a single line, to fit into a property
-	      (setq note (mapconcat 'identity
-				    (org-split-string (org-trim note) "\n")
-				    "\\n")))
+	  (setq old (org-string-nw-p old))
+	  (setq new (org-string-nw-p new))
 	  (unless (equal data "body")
-	    (setq new (and new (org-trim new))
-		  old (and old (org-trim old))))
+	    (setq new (and new (org-trim new)))
+	    (setq old (and old (org-trim old))))
 	  (goto-char (+ 2 bos-marker))
 	  ;; Remember this place so that we can return
 	  (move-marker marker (point))
 	  (setq org-mobile-error nil)
-	  (save-excursion
-	    (condition-case msg
-		(org-with-point-at id-pos
-		  (progn
-		    (eval cmd)
-		    (unless (member data (list "delete" "archive" "archive-sibling" "addheading"))
-		      (if (member "FLAGGED" (org-get-tags))
-			  (add-to-list 'org-mobile-last-flagged-files
-				       (buffer-file-name (current-buffer)))))))
-	      (error (setq org-mobile-error msg))))
+	  (condition-case msg
+	      (org-with-point-at id-pos
+		(funcall cmd data old new)
+		(unless (member data '("delete" "archive" "archive-sibling"
+				       "addheading"))
+		  (when (member "FLAGGED" (org-get-tags nil t))
+		    (add-to-list 'org-mobile-last-flagged-files
+				 (buffer-file-name)))))
+	    (error (setq org-mobile-error msg)))
 	  (when org-mobile-error
-	    (org-pop-to-buffer-same-window (marker-buffer marker))
+	    (pop-to-buffer-same-window (marker-buffer marker))
 	    (goto-char marker)
-	    (incf cnt-error)
+	    (cl-incf cnt-error)
 	    (insert (if (stringp (nth 1 org-mobile-error))
 			(nth 1 org-mobile-error)
 		      "EXECUTION FAILED")
@@ -923,8 +903,8 @@ If BEG and END are given, only do this in that region."
     (save-buffer)
     (move-marker marker nil)
     (move-marker end nil)
-    (message "%d new, %d edits, %d flags, %d errors" cnt-new
-	     cnt-edit cnt-flag cnt-error)
+    (message "%d new, %d edits, %d flags, %d errors"
+	     cnt-new cnt-edit cnt-flag cnt-error)
     (sit-for 1)))
 
 (defun org-mobile-timestamp-buffer (buf)
@@ -1019,7 +999,7 @@ be returned that indicates what went wrong."
        ((equal new "DONEARCHIVE")
 	(org-todo 'done)
 	(org-archive-subtree-default))
-       ((equal new current) t) ; nothing needs to be done
+       ((equal new current) t)		; nothing needs to be done
        ((or (equal current old)
 	    (eq org-mobile-force-mobile-change t)
 	    (memq 'todo org-mobile-force-mobile-change))
@@ -1028,7 +1008,7 @@ be returned that indicates what went wrong."
 		 old current))))
 
      ((eq what 'tags)
-      (setq current (org-get-tags)
+      (setq current (org-get-tags nil t)
 	    new1 (and new (org-split-string new ":+"))
 	    old1 (and old (org-split-string old ":+")))
       (cond
@@ -1036,38 +1016,42 @@ be returned that indicates what went wrong."
        ((or (org-mobile-tags-same-p current old1)
 	    (eq org-mobile-force-mobile-change t)
 	    (memq 'tags org-mobile-force-mobile-change))
-	(org-set-tags-to new1) t)
+	(org-set-tags new1) t)
        (t (error "Tags before change were expected as \"%s\", but are \"%s\""
 		 (or old "") (or current "")))))
 
      ((eq what 'priority)
-      (when (looking-at org-complex-heading-regexp)
-	(setq current (and (match-end 3) (substring (match-string 3) 2 3)))
-	(cond
-	 ((equal current new) t) ; no action required
-	 ((or (equal current old)
-	      (eq org-mobile-force-mobile-change t)
-	      (memq 'tags org-mobile-force-mobile-change))
-	  (org-priority (and new (string-to-char new))))
-	 (t (error "Priority was expected to be %s, but is %s"
-		   old current)))))
+      (let ((case-fold-search nil))
+	(when (looking-at org-complex-heading-regexp)
+	  (let ((current (and (match-end 3) (substring (match-string 3) 2 3))))
+	    (cond
+	     ((equal current new) t)	;no action required
+	     ((or (equal current old)
+		  (eq org-mobile-force-mobile-change t)
+		  (memq 'tags org-mobile-force-mobile-change))
+	      (org-priority (and new (string-to-char new))))
+	     (t (error "Priority was expected to be %s, but is %s"
+		       old current)))))))
 
      ((eq what 'heading)
-      (when (looking-at org-complex-heading-regexp)
-	(setq current (match-string 4))
-	(cond
-	 ((equal current new) t) ; no action required
-	 ((or (equal current old)
-	      (eq org-mobile-force-mobile-change t)
-	      (memq 'heading org-mobile-force-mobile-change))
-	  (goto-char (match-beginning 4))
-	  (insert new)
-	  (delete-region (point) (+ (point) (length current)))
-	  (org-set-tags nil 'align))
-	 (t (error "Heading changed in MobileOrg and on the computer")))))
+      (let ((case-fold-search nil))
+	(when (looking-at org-complex-heading-regexp)
+	  (let ((current (match-string 4)))
+	    (cond
+	     ((equal current new) t)	;no action required
+	     ((or (equal current old)
+		  (eq org-mobile-force-mobile-change t)
+		  (memq 'heading org-mobile-force-mobile-change))
+	      (goto-char (match-beginning 4))
+	      (insert new)
+	      (delete-region (point) (+ (point) (length current)))
+	      (org-align-tags))
+	     (t
+	      (error
+	       "Heading changed in the mobile device and on the computer")))))))
 
      ((eq what 'addheading)
-      (if (org-at-heading-p) ; if false we are in top-level of file
+      (if (org-at-heading-p)	; if false we are in top-level of file
 	  (progn
 	    ;; Workaround a `org-insert-heading-respect-content' bug
 	    ;; which prevents correct insertion when point is invisible
@@ -1082,7 +1066,7 @@ be returned that indicates what went wrong."
      ((eq what 'refile)
       (org-copy-subtree)
       (org-with-point-at (org-mobile-locate-entry new)
-	(if (org-at-heading-p) ; if false we are in top-level of file
+	(if (org-at-heading-p)	; if false we are in top-level of file
 	    (progn
 	      (setq level (org-get-valid-level (funcall outline-level) 1))
 	      (org-end-of-subtree t t)
@@ -1117,7 +1101,8 @@ be returned that indicates what went wrong."
 					(outline-next-heading)
 					(point))))
 	t)
-       (t (error "Body was changed in MobileOrg and on the computer")))))))
+       (t (error
+	   "Body was changed in the mobile device and on the computer")))))))
 
 (defun org-mobile-tags-same-p (list1 list2)
   "Are the two tag lists the same?"
